from moneywave import MoneyWave
from pprint import PrettyPrinter

pp = PrettyPrinter(indent=3)

ACCESS_BANK = "0690000004"
PROVIDUS_BANK = 5900102340
UNITY_BANK = "0025756133"
OTP_BANK = 12345

API_KEY = "YOUR_API_KEY"
SECRET = "YOUR_SECRET_KEY"
CARDS = {
    "verve": {'no': "5061020000000000094", 'cvv': 347, 'exp_m': "07",
              'exp_y': 20, 'pin': 111, 'otp': 123456},
    "visa": {'no': "4242424242424242", 'cvv': 812, 'exp_m': "01", 'exp_y': 19,
             'pin': 3310, 'otp': 12345},
    "mastercard": {'no': '5438898014560229', 'cvv': 789, 'exp_m': "09",
                   'exp_y': 19, 'pin': 3310, 'otp': 12345},
    "visa_local": {'no': '4187427415564246', 'cvv': 828, 'exp_m': '09',
                   'exp_y': 19, 'pin': 3310, 'otp': 12345},
    "visa_international": {'no': '4556052704172643', 'cvv': 899, 'exp_m': '01',
                           'exp_y': 19}
}
TEST_CARD_VERVE = "5061020000000000094"
TEST_CARD_VISA = "4242424242424242"

EXP_M = "07"
EXP_Y = "20"
CVV = 347
PIN = 111
OTP = 123456

API = MoneyWave(API_KEY, SECRET)
"""
print("======== GET BALANCE ========")
r = API.Wallet.get_balance()
pp.pprint(r)

print("======= CREATE SUB WALLET =======")
# r = API.Wallet.create_sub_wallet("Johnny", "sayWHAAATT!",
# "cus_0399292", "NGN")
# pp.pprint(r)


print("======= CREATE CARD ========")
r = API.Resource.create_card(TEST_CARD_VERVE, EXP_M, EXP_Y, CVV)
pp.pprint(r)

print("======= CARD INFOS ========")
r = API.Resource.card_details(TEST_CARD_VERVE)
pp.pprint(r)


print("======== CREDIT WALLET =======")
rr = API.Wallet.funding.card_to_wallet("John", "Doe", "+2348122681468",
                                       "johnny.deezy@gmail.com", "c_0093934nk",
                                       CARDS.get('visa').get('no'),
                                       CARDS.get('visa').get('exp_m'),
                                       CARDS.get('visa').get('exp_y'),
                                       CARDS.get('visa').get('cvv'), 300, 35,
                                       "https://google.fr", "mobile")
pp.pprint(rr)
"""


from moneywave.wallet.funding import Funding
from moneywave.wallet.transfer import Transfer


class Wallet:
    def __init__(self, util):
        self.util = util
        self.funding = Funding(util)
        self.transfer = Transfer(util)

    def get_balance(self):
        return self.util.send_request(self.util.settings.get_balance, None, method="get")

    def create_sub_wallet(self, name, lock_code, user_ref, currency):
        data = {"name": name, "lock_code": lock_code, "user_ref": user_ref,
                'currency': currency}
        return self.util.send_request(self.util.settings.create_wallet, data)


class Account:
    def __init__(self, util):
        self.util = util

    def card_to_account(self, first_name, last_name, phone_number, email,
                        recipient_bank_account, recipient_bank_code,
                        card_no, exp_year, exp_month, cvv,
                        amount, fee, redirect_url, medium="mobile",
                        narration=None, pin=None):
        is_verve, is_local_mastercard = self.check_card_type(card_no)

        data = {
            "firstname": first_name, "lastname": last_name,
            "phonenumber": phone_number, "recipient": "account",
            "email": email, "card_no": card_no, "cvv": cvv,
            "expiry_month": exp_month, "expiry_year": exp_year,
            "recipient_bank": recipient_bank_code,
            "recipient_account_number": recipient_bank_account,
            "amount": amount, "fee": fee, "medium": medium,
            "redirecturl": redirect_url
        }
        if is_verve:
            if pin:
                data["pin"] = pin
            else:
                raise Exception('VERVE Cards required pin')
        elif is_local_mastercard:
            data["charge_with"] = "card"
            data["charge_auth"] = "PIN"
        if narration:
            data["narration"] = narration
        response = self.util.send_request(self.util.settings.account_transfer,
                                          data)
        return response

    def card_to_account_validation(self, transaction_ref, otp):
        data = {'transactionRef': transaction_ref, 'otp': otp}
        response = self.util.send_request(
            self.util.settings.account_verification, data)
        return response

    def check_card_type(self, card_no):
        response = self.util.send_request(self.util.settings.card_enquiry,
                                          {"cardNumber": card_no})
        is_verve, is_local_mastercard = False, False
        if 'data' in response:
            if response.get('data').get('cardType') == "VERVE":
                is_verve = True
            elif response.get('data').get(
                    'cardType') == "LOCAL" and response.get('data').get(
                    'cardBrand') == "MASTERCARD":
                is_local_mastercard = True

        return is_verve, is_local_mastercard



class Transfer:
    def __init__(self, util):
        self.util = util

    def to_wallet(self, source_wallet, recipient_wallet, amount, currency,
                  lock):
        data = {"sourceWallet": source_wallet,
                "recipientWallet": recipient_wallet,
                "amount": amount, "currency": currency, "lock": lock}
        return self.util.send_request(self.util.settings.transfer_wallet, data)

    def to_account(self, sender_name, ref, account_number,
                   bank_code, amount, currency, lock, narration=None):
        data = {"senderName": sender_name, "accountNumber": account_number,
                "bankcode": bank_code, "amount": amount, "currency": currency,
                "ref": ref, "lock": lock}
        if narration:
            data['narration'] = narration
        return self.util.send_request(self.util.settings.transfer_account, data)

    def to_account_bulk(self, sender_name, ref, name, recipients, lock,
                        currency=None):
        if not isinstance(recipients):
            raise TypeError('recipients should be a list')
        else:
            valid = [True for x in recipients if (
            'amount' in x and 'bankcode' in x and 'accountNumber' in x)]
            if len(valid) != len(recipients):
                raise ValueError(
                    'recipents objects should be {"amount": amount, '
                    '"bankcode": bankcode, "accountNumber": account_number}')

        data = {'senderName': sender_name, 'name': name, 'lock': lock,
                'ref': ref,
                'recipients': recipients, 'instantQueue': True}
        if currency:
            data['currency'] = currency

        return self.util.send_request(self.util.settings.transfer_account_bulk,
                                      data)



charge_response_codes = {
    "0": "Successful",
    "00": "Successful",
    "0-M": "Verification attempted",
    "0-Y": "Verification Successful",
    "RR-V": "Transaction already validated.",
    "02": "Needs card Validation",
    "RR": "Transaction Failed. Detailed Message is included in response message",
    "2": "Declined",
    "7": "Card Declined due to invalid card data",
    "RR-T2": "Card not enrolled for safe token, user should contact their bank",
    "XS0": "Authorization Failed due to connectivity issues with the bank",
    "B01": "Invalid BVN",
    "RR-51": "Insufficient Funds",
    "RR-R3": "CardToken is mandatory!",
    "RR-14": "Invalid Card Number",
    "RR-55": "Incorrect PIN",
    "R0": "Transaction Failed due to connectivity issues with the bank",
    "RR-E42": "Card Declined due to invalid card expiry",
    "RR-56": "No Card Record",
    "RR-2": "Card Declined",
    "RR-X04": "Transaction Amount too low",
    "RR-15": "Transaction error",
    "RR-7": "Card Declined due to invalid card security code",
    "RR-57": "Transaction not Permitted to Cardholder",
    "RR-04": "Pick-up card",
    "RR-Z8": "Payment Gateway currently does not accept your card type",
    "RR-91": "Bank or switch network error",
    "EEE": "An unexpected error occurred!",
    "RR-E18": "The service provider is unreachable at the moment, please try "
              "again later.",
    "RR-E19": "An invalid response was received from remote host, see provider "
              "response code/message for details.",
    "RR-E32": "JSON is badly formatted or it contains invalid character.",
    "RR-E57": "The PIN contains an invalid character",
    "RR-EE4": "Card Details could not be Retrieved!",
    "RR-R401": "Card has been blocked due to too many failed retries.",
    "N-E": "Card not enrolled for 3DSecure",
    "BR0": "Timeout on BVN check",
    "RN0": "Invalid Account"
}

disburse_response_codes = {
    "400": "Bad Request – Your request sucks",
    "401": "Unauthorized – Your API key is wrong",
    "403": "Forbidden – The endpoint requested is hidden for administrators"
           " only",
    "404": "Not Found – The specified endpoint could not be found",
    "405": "Method Not Allowed – You tried to access an endpoint with an "
           "invalid method",
    "406": "Not Acceptable – You requested a format that isn’t json",
    "410": "Gone – The endpoint requested has been removed from our servers",
    "429": "Too Many Requests!",
    "500": "Internal Server Error – We had a problem with our server. Try again"
           " later.",
    "503": "Service Unavailable – We’re temporarily offline for maintenance."
           " Please try again later."
}


class Settings:
    def __init__(self, api_key, secret_key, mode):
        self.api_key = api_key
        self.secret_key = secret_key

        if mode == "test":
            self.url = "https://moneywave.herokuapp.com"
        else:
            self.url = "https://live.moneywaveapi.co/"
        self.access_token = "/v1/merchant/verify"
        self.account_transfer = "/v1/transfer"
        self.account_charge_card = "/v1/transfer/charge/auth/card"
        self.funding_pay = "/v1/transfer"
        self.funding_charge_account = "/v1/transfer/charge/auth/account"
        self.funding_charge_card = self.funding_pay
        self.transfer_wallet = "/v1/wallet/transfer"
        self.transfer_account = "/v1/disburse"
        self.transfer_account_bulk = "/v1/disburse/queue"
        self.resource_banks = "/v1/banks"
        self.account_verification = "/v1/resolve/account"
        self.card_tokenization = "/v1/transfer/charge/tokenize/card"
        self.card_enquiry = "/v1/user/card/check"
        self.reporting = "/v1/report/transactions"
        self.transaction_get_total = "/v1/get-charge"
        self.transaction_retrial = "/v1/transfer/disburse/retry"
        self.transaction_previous_card = "/v1/transfer/{0}"
        self.transaction_status = "/v1/transfer/charge/status"
        self.transaction_wallet = "/v1/disburse/status"
        self.get_balance = "/v1/wallet"
        self.create_wallet = self.get_balance

    # noinspection PyMethodMayBeStatic
    def get_error_msg(self, code):
        return charge_response_codes.get(code) or disburse_response_codes.get(
            code) or 'Unknown Error'



from moneywave.account import Account
from moneywave.resources import Resource
from moneywave.transaction import Transaction
from moneywave.utils import Util, Settings
from moneywave.wallet import Wallet


class MoneyWave:
    def __init__(self, api_key, secret_key, mode="test"):
        self.settings = Settings(api_key, secret_key, mode)
        self.__util = Util(self.settings)
        self.Account = Account(self.__util)
        self.Resource = Resource(self.__util)
        self.Wallet = Wallet(self.__util)
        self.Transaction = Transaction(self.__util)

    def __get_auth_key(self):
        pass

    @property
    def token(self):
        return self.__util.token



from datetime import datetime


class Resource:
    def __init__(self, util):
        self.util = util

    def get_banks(self):
        return self.util.send_request(self.util.settings.resource_banks, {})

    def validate_account(self, account_number, bank_code):
        data = {"account_number": account_number, "bank_code": bank_code}
        return self.util.send_request(self.util.settings.account_verification,
                                      data)

    def create_card(self, card_number, expiry_month, expiry_year, cvv):
        data = {"card_no": card_number, "expiry_year": expiry_year,
                "expiry_month": expiry_month, "cvv": cvv}
        return self.util.send_request(self.util.settings.card_tokenization,
                                      data)

    def card_details(self, card_number):
        data = {"cardNumber": card_number}
        return self.util.send_request(self.util.settings.card_enquiry, data)

    def get_reports(self, status, amount, flutterwave_ref, ref, currency="NGN",
                    date=datetime.today(), limit=None, page=None,
                    type="credit"):
        data = {"status": status, "amount": amount, "flwref": flutterwave_ref,
                "ref": ref, "currency": currency, "date": date, "type": type}
        if limit:
            data['limit'] = limit
        if page:
            data['page'] = page

        return self.util.send_request(self.util.settings.reporting, data)



import base64
import json

import requests
from Crypto.Cipher import DES3


class Util:
    def __init__(self, settings):
        self.settings = settings
        self.__get_auth()

    def __get_auth(self):
        headers = {"Accept": "application/json"}

        response = requests.post(self.resolve_url(self.settings.access_token),
                                 headers=headers,
                                 data={"apiKey": self.settings.api_key,
                                       'secret': self.settings.secret_key}
                                 )
        if response.status_code == 200:
            body = response.json()
            if 'token' in body:
                self.token = body.get('token')
        else:
            self.token = None

            raise Exception(self.settings.get_error_msg(response.status_code),
                            response.json())

    def resolve_url(self, endpoint):
        return self.settings.url + endpoint

    def send_request(self, endpoint, data, method="post"):
        headers = {"Accept": "application/json",
                   "Authorization": self.token}
        if method == "get":
            req = requests.get(self.resolve_url(endpoint), headers=headers)
        else:
            req = requests.post(self.resolve_url(endpoint), data=data,
                                headers=headers)
        if req.status_code == 200:
            body = req.json()
            if 'status' in body:
                if body.get('status') == "success":
                    if 'data' in body:
                        return body.get('data')
                    else:
                        return body
                else:
                    return {'status': 'error',
                            'message': self.settings.get_error_msg(
                                body.get('code')),
                            "response": req.json()}
            return
        else:
            return {"status": "error",
                    "message": self.settings.get_error_msg(req.status_code),
                    "response": req.json()}

    def encrypt(self, message):

        block_size = 8
        padding_size = block_size - (len(message) % block_size)

        if padding_size > 0:
            message += chr(padding_size) * padding_size
        cipher = DES3.new(message, DES3.MODE_ECB)
        ciphered = cipher.encrypt(message.encode('utf-8'))
        return base64.b64encode(ciphered).decode('utf-8')



from distutils.core import setup

setup(
    name='moneywave',
    version='v1.1',
    packages=['moneywave', 'moneywave.utils', 'moneywave.wallet',
              'moneywave.wallet.funding', 'moneywave.wallet.transfer',
              'moneywave.account', 'moneywave.resources',
              'moneywave.transaction'],
    url='https://github.com/alekaizer/moneywave-python',
    license='MIT',
    author='Achille AROUKO',
    author_email='achille.arouko@gmail.com',
    description='MoneyWave Python Library'
)



class Funding:
    def __init__(self, util):
        self.util = util

    def pay_with_internet_bank(self, first_name, last_name, phone_number, email,
                               recipient_id, sender_bank, amount,
                               medium, redirect_url):

        data = {"firstname": first_name, "lastname": last_name, "email": email,
                "phonenumber": phone_number, "recipient": "wallet",
                "recipient_id": recipient_id, "amount": amount,
                "sender_bank": sender_bank,
                "medium": medium, "redirecturl": redirect_url,
                "charge_auth": "INTERNETBANKING",
                "charge_with": "ext_account",
                "apiKey": self.util.settings.api_key}
        return self.util.send_request(self.util.settings.funding_pay, data)

    def account_to_wallet(self, first_name, last_name, phone_number, email,
                          sender_account_number, sender_bank, amount,
                          redirect_url):

        data = {"firstname": first_name, "lastname": last_name, "email": email,
                "phonenumber": phone_number, "recipient": "wallet",
                "sender_account_number": sender_account_number,
                "amount": amount, "sender_bank": sender_bank,
                "redirecturl": redirect_url, "charge_auth": "INTERNETBANKING",
                "charge_with": "account", "apiKey": self.util.settings.api_key}

        return self.util.send_request(self.util.settings.account_transfer, data)

    def account_to_wallet_validation(self, transaction_ref, otp=None,
                                     account_credit=None):
        if not (otp or account_credit):
            raise Exception('OTP or ACCOUNT_CREDIT value is required')
        else:
            auth_type = "OTP" if otp else "ACCOUNT_CREDIT"
            auth_value = otp or account_credit
        data = {"transactionRef": transaction_ref, "authType": auth_type,
                "authValue": auth_value}

        return self.util.send_request(self.util.settings.funding_charge_account,
                                      data)

    def card_to_wallet(self, first_name, last_name, phone_number, email,
                       recipient_id, card_no, expiry_month,
                       expiry_year, cvv, amount, fee, redirect_url, medium,
                       narration=None, pin=None):
        is_verve, is_mastercard = self.check_card_type(card_no)
        if is_verve and not pin:
            raise Exception('Verve cards require PIN value, please set the '
                            'argument')
        data = {"firstname": first_name, "lastname": last_name, "email": email,
                "phonenumber": phone_number, "recipient": "wallet",
                "recipient_id": recipient_id, "card_no": card_no,
                "expiry_month": expiry_month, "expiry_year": expiry_year,
                "cvv": cvv, "apiKey": self.util.settings.api_key,
                "medium": medium, "charge_with": "card",
                "fee": fee, "amount": amount, "redirecturl": redirect_url}
        if narration:
            data['narration'] = narration
        if is_verve:
            data['pin'] = pin
        if is_mastercard:
            data['charge_with'] = "card"
            data['charge_auth'] = 'PIN'

        return self.util.send_request(self.util.settings.funding_pay, data)

    def card_to_wallet_with_token(self, first_name, last_name, phone_number,
                                  email, recipient_id, card_token, pin, amount,
                                  fee, redirect_url, medium):

        data = {"firstname": first_name, "lastname": last_name, "email": email,
                "phonenumber": phone_number, "recipient": "wallet", "pin": pin,
                "recipient_id": recipient_id, "card_token": card_token,
                "apiKey": self.util.settings.api_key, "medium": medium,
                "charge_with": "tokenized_card", "fee": fee, "amount": amount,
                "redirecturl": redirect_url}

        return self.util.send_request(self.util.settings.funding_pay, data)

    def card_details(self, card_number):
        data = {"cardNumber": card_number}
        return self.util.send_request(self.util.settings.card_enquiry, data)

    def check_card_type(self, card_no):
        response = self.util.send_request(self.util.settings.card_enquiry,
                                          {"cardNumber": card_no})
        is_verve, is_local_mastercard = False, False
        if 'data' in response:
            print("CARD INFOS", response.get('data'))
            if response.get('data').get('cardBrand') == "MAESTRO":
                is_verve = True
            elif response.get('data').get(
                    'countryOfIssueISO') == "NG" and response.get('data').get(
                'cardBrand') == "MASTERCARD":
                is_local_mastercard = True

        return is_verve, is_local_mastercard



class Transaction:
    def __init__(self, util):
        self.util = util

    def get_total_amount(self, amount, merchant_fee=0):
        data = {"amount": amount, "fee": merchant_fee}
        return self.util.send_request(self.util.settings.transaction_get_total,
                                      data)

    def retry_disburse(self, charge_id, recipient_account_number,
                       recipient_bank):
        data = {"id": charge_id,
                "recipient_account_number": recipient_account_number,
                "recipient_bank": recipient_bank}
        return self.util.send_request(self.util.settings.transaction_retrial,
                                      data)

    def get_transaction(self, transaction_id=None, ref=None, source="card"):
        if source == "card":
            if not transaction_id:
                raise Exception('Missing transaction_id')
            else:
                response = self.util.send_request(
                    self.util.settings.transaction_previous_card.format(
                        transaction_id), {})
        elif source == "wallet":
            if not ref:
                raise Exception('Missing ref')
            else:
                data = {'ref': ref}
                response = self.util.send_request(
                    self.util.settings.transaction_wallet, data)
        else:
            raise Exception('Unknown source. '
                            'Source can be either "card" or "wallet"')
        return response

    def transaction_status(self, ref):
        return self.util.send_request(self.util.settings.transaction_status,
                                      {"ref": ref})
